/**
 * EasyBeans
 * Copyright (C) 2006-2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AbstractInjectionHandler.java 4699 2009-02-24 14:07:04Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.annotation.processor.handler;

import org.ow2.util.annotation.processor.IAnnotationHandler;
import org.ow2.util.annotation.processor.ProcessorException;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * An {@link AbstractInjectionHandler} is a simple {@link IAnnotationHandler}
 * that provides facility methods to perform field and method injection.
 *
 * @author Guillaume Sauthier
 */
public abstract class AbstractInjectionHandler extends AbstractAnnotationHandler {

    /**
     * Inject a given value in the given object, using the 'setter'
     * method provided as parameter.
     *
     * @param method setter method
     * @param target instance to be injected
     * @param value  value to be injected
     * @throws ProcessorException
     */
    protected void doInjectMethod(final Method method,
                                  final Object target,
                                  final Object value) throws ProcessorException {
        boolean accessibility = method.isAccessible();

        try {
            method.setAccessible(true);
            method.invoke(target, value);
        } catch (IllegalArgumentException e) {
            throw new ProcessorException("Cannot inject '" + method + "'", e);
        } catch (IllegalAccessException e) {
            throw new ProcessorException("Cannot inject '" + method + "'", e);
        } catch (InvocationTargetException e) {
            throw new ProcessorException("Cannot inject '" + method + "'", e);
        } finally {
            method.setAccessible(accessibility);
        }
    }

    /**
     * Inject a given value in the given object, using the field provided
     * as parameter.
     *
     * @param field  injection field
     * @param target instance to be injected
     * @param value  value to be injected
     * @throws ProcessorException
     */
    protected void doInjectField(final Field field,
                                 final Object target,
                                 final Object value) throws ProcessorException {
        boolean accessibility = field.isAccessible();

        try {
            field.setAccessible(true);
            field.set(target, value);
        } catch (IllegalArgumentException e) {
            throw new ProcessorException("Cannot inject '" + field + "'", e);
        } catch (IllegalAccessException e) {
            throw new ProcessorException("Cannot inject '" + field + "'", e);
        } finally {
            field.setAccessible(accessibility);
		}
	}

    /**
     * Compute the attribute name from a 'setter' method. Recognized patterns
     * are:
     * <ul>
     * <li>setAttribute()</li>
     * <li>unsetAttribute()</li>
     * <li>bindAttribute()</li>
     * <li>unbindAttribute()</li>
     * </ul>
     * <p/> The attribute name is then lower-cased (first char).
     * @param method base method
     * @return the attribute name (Java Bean property name)
     * @throws org.ow2.util.annotation.processor.ProcessorException if the method do not follow the recognized
     *         patterns
     */
    protected String getAttributeName(final Method method) throws ProcessorException {
        String name = method.getName();
        String attributeName = null;
        if (name.startsWith("set")) {
            attributeName = name.substring(3);
        } else if (name.startsWith("unset")) {
            attributeName = name.substring(5);
        } else if (name.startsWith("bind")) {
            attributeName = name.substring(4);
        } else if (name.startsWith("unbind")) {
            attributeName = name.substring(6);
        } else {
            throw new ProcessorException("Method '" + method + "' is not a recognised method");
        }

        attributeName = attributeName.substring(0, 1).toLowerCase() + attributeName.substring(1);

        return attributeName;
    }
    
    /**
     * @param method Method under test
     * @return <code>true</code> only if the given method is a setter method.
     */
    protected boolean isSetterMethod(final Method method) {
        
        // A setter method's name starts with "set"
        String name = method.getName();
        if (!name.startsWith("set")) {
            return false;
        }
        
        // A setter method only have 1 parameter
        Class<?>[] types = method.getParameterTypes();
        if (types.length != 1) {
            return false;
        }
        
        // Otherwise, that's a setter
        return true;
        
    }

    /**
     * @param method Method under test
     * @return the type of the setter method
     */
    protected Class<?> getSetterParameterType(final Method method) {

        if (!isSetterMethod(method)) {
            throw new IllegalStateException("Method " + method + " is not a setter method");
        }
        
        // A setter method only have 1 parameter
        return method.getParameterTypes()[0];
    }
}
